#!/usr/bin/env python3

import time
from datetime import datetime
import requests
import traceback
from influxdb import InfluxDBClient

OUT = "/data/log.csv"
URL = "http://62.220.135.196:8080/sensor/{}?version=1.0&interval=minute&unit=watt"
SENSORS = (
  "34cde81adabfb1ce819eca8fea6949b6",
  "b7755b5f3ec05fcdc67f449241a9912a",
  "e67e0685f747b30d855108ab781abdfc"
)

print("Starting logger")
db = InfluxDBClient('influxdb', database='power')
print("Connected to %s" % db)

while True:
  points = []
  last_timestamps = {}

  for sensor in SENSORS:
    try:
      last_timestamp = last_timestamps.get(sensor, 0)
      flukso = requests.get(URL.format(sensor)).json()
      print("Before filtering: " + str(len(flukso)))

      # Remove NaN values
      flukso = filter(lambda x: x[1] != 'nan', flukso)

      # Remove values that were already seen before
      flukso = list(filter(lambda x: x[0] >= last_timestamp, flukso))

      if len(flukso) > 0:
        last_timestamp = flukso[-1][0]

      print("After filtering: " + str(len(flukso)))

      for timestamp, value in flukso:
        points.append((timestamp, sensor, value))
    except Exception as e:
      traceback.print_exc()

  try:
    db.write_points([{
             "measurement": "flukso",
             "time": datetime.fromtimestamp(timestamp).isoformat(),
             "tags": {
                "sensor": sensor,
             },
             "fields": { "watts": float(value) }
           } for timestamp, sensor, value in points])
    print("Wrote %d samples to influxdb" % len(points))
  except Exception as e:
    print("Write to influxdb failed")
    traceback.print_exc()

  try:
    with open(OUT, 'a') as f:
      for timestamp, sensor, value in points:
        f.write(",".join((str(timestamp), sensor, str(value))) + "\n")
    print("Wrote %d samples to csv" % len(points))
  except Exception as e:
    print("Write to csv file failed")
    traceback.print_exc()

  sleeptime = 45 * 1000000 - datetime.utcnow().microsecond
  print("Sleeping %d us" % sleeptime)
  time.sleep(sleeptime/1000000.0)
